@@ -16,6 +16,8 @@ from .err import \
     PasswordPolicyChangeAfterReset, \
     PasswordPolicyExpirationWarning, \
     PasswordPolicyExpiredError
+from . import LIBLDAP_TLS_PACKAGE
+from ldap0.controls.simple import AuthorizationIdentityResponseControl
 from .err import NoUniqueEntry
 from .base import encode_list
 from .dn import DNObj
@@ -35,6 +37,7 @@ from .sasl import SaslNoninteractiveAuth
 from .modlist import add_modlist, modify_modlist
 from .ldapurl import LDAPUrl
 from .typehints import AttrList, EntryMixed, EntryStr, RequestControls
+from .cache import Cache
 
 if __debug__:
     # Tracing is only supported in debugging mode
@@ -62,9 +65,7 @@ class LDAPObject:
     """
     __slots__ = (
         '_cache',
-        '_cache_hits',
-        '_cache_misses',
-        'cache_ttl',
+        '_cache_ttl',
         'encoding',
         '_l',
         '_libldap0_lock',
@@ -115,7 +116,8 @@ class LDAPObject:
         self._whoami_dn = None
         self.timeout = -1
         self.protocol_version = _libldap0.VERSION3
-        self.cache_ttl = cache_ttl
+        self._cache_ttl = cache_ttl
+        self._cache = Cache(ttl=cache_ttl)
         self.flush_cache()
         self.set_option(_libldap0.OPT_RESTART, False)
         self.set_option(_libldap0.OPT_DEREF, False)
@@ -230,6 +232,7 @@ class LDAPObject:
             client_cert_filename=None,
             client_key_filename=None,
             req_cert=_libldap0.OPT_X_TLS_DEMAND,
+            crl_check=_libldap0.OPT_X_TLS_CRL_NONE,
         ):
         """
         set TLS options for connection even with checking whether cert/key
@@ -253,6 +256,9 @@ class LDAPObject:
                     raise
         # Force server cert validation
         self._l.set_option(_libldap0.OPT_X_TLS_REQUIRE_CERT, req_cert)
+        # CRL check
+        if LIBLDAP_TLS_PACKAGE == 'OpenSSL':
+            self._l.set_option(_libldap0.OPT_X_TLS_CRLCHECK, crl_check)
         # this has to be the last option set to let libldap reinitialize TLS context
         self._l.set_option(_libldap0.OPT_X_TLS_NEWCTX, 0)
         # end of set_tls_options()
@@ -332,8 +338,9 @@ class LDAPObject:
         return res
 
     def simple_bind(
-            self, who='',
-            cred=b'',
+            self,
+            who: str = '',
+            cred: bytes = b'',
             req_ctrls: Optional[RequestControls] = None,
         ) -> int:
         self.flush_cache()
@@ -344,16 +351,28 @@ class LDAPObject:
             encode_request_ctrls(req_ctrls),
         )
 
+    def _handle_authzid_ctrl(self, bind_ctrls):
+        authz_id_ctrls = [
+            ctrl
+            for ctrl in bind_ctrls
+            if ctrl.controlType == AuthorizationIdentityResponseControl.controlType
+        ]
+        if authz_id_ctrls and len(authz_id_ctrls) == 1:
+            authz_id = authz_id_ctrls[0].authzId.decode('utf-8')
+            if authz_id.startswith('dn:'):
+                self._whoami_dn = authz_id[3:]
+
     def simple_bind_s(
             self,
-            who='',
-            cred=b'',
+            who: str = '',
+            cred: bytes = b'',
             req_ctrls: Optional[RequestControls] = None,
         ) -> LDAPResult:
         msgid = self.simple_bind(who, cred, req_ctrls)
         res = self.result(msgid, _libldap0.MSG_ALL, self.timeout)
         assert res.rtype == _libldap0.RES_BIND, ValueError('Wrong result type %d' % (res.rtype,))
         if res.ctrls:
+            self._handle_authzid_ctrl(res.ctrls)
             # Extract the password policy response control and raise
             # appropriate exceptions if needed
             ppolicy_ctrls = [
@@ -849,22 +868,19 @@ class LDAPObject:
         """
         Returns percentage of cache hit ratio
         """
-        try:
-            return (
-                100 * float(self._cache_hits) /
-                float(self._cache_hits+self._cache_misses)
-            )
-        except ZeroDivisionError:
-            return 0.0
+        return self._cache.hit_ratio
 
     def flush_cache(self) -> None:
         """
         reset/flush the internal cache
         """
+        try:
+            cache = self._cache
+        except AttributeError:
+            pass
+        else:
+            cache.flush()
         self._whoami_dn = None
-        self._cache_misses = 0
-        self._cache_hits = 0
-        self._cache = {}
 
     def uncache(self, entry_dn) -> None:
         """
@@ -873,12 +889,17 @@ class LDAPObject:
         entry_dn = entry_dn.lower()
         remove_keys = set()
         # first find the items to be removed from cache
-        for cache_key_args, cached_results in self._cache.items():
-            cached_results = self._cache[cache_key_args]
+        for cache_key_args in list(self._cache):
             if cache_key_args[0].lower() == entry_dn:
                 remove_keys.add(cache_key_args)
+                continue
+            try:
+                cached_results = self._cache[cache_key_args]
+            except KeyError:
+                # cached items expired in between
+                pass
             else:
-                for cached_res in cached_results[1]:
+                for cached_res in cached_results:
                     if (
                             isinstance(cached_res, SearchResultEntry)
                             and cached_res.dn_s.lower() == entry_dn
@@ -905,9 +926,8 @@ class LDAPObject:
             cache_ttl: Optional[Union[int, float]] = None,
         ) -> List[LDAPResult]:
         if cache_ttl is None:
-            cache_ttl = self.cache_ttl
+            cache_ttl = self._cache._ttl
         if cache_ttl > 0:
-            current_time = time.time()
             cache_key_args = (
                 base,
                 scope,
@@ -923,18 +943,12 @@ class LDAPObject:
             )
             # first look into cache for non-expired search results
             try:
-                result_cache_time, res = self._cache[cache_key_args]
+                res = self._cache[cache_key_args]
             except KeyError:
                 pass
             else:
-                if current_time > result_cache_time + cache_ttl:
-                    # cache entry is expired => remove it no matter what happens later
-                    del self._cache[cache_key_args]
-                else:
-                    self._cache_hits += 1
-                    return res
+                return res
         # no cached result
-        self._cache_misses += 1
         msgid = self.search(
             base,
             scope,
@@ -950,11 +964,9 @@ class LDAPObject:
             res.extend(ldap_res.rdata)
         if cache_ttl > 0:
             # store result in cache if caching is enabled at all
-            self._cache[cache_key_args] = (
-                current_time,
-                res
-            )
-        return res # search_s()
+            self._cache.cache(cache_key_args, res, cache_ttl)
+        return res
+        # search_s()
 
     def noop_search(
             self,
@@ -1064,13 +1076,13 @@ class LDAPObject:
 
     def read_s(
             self,
-            dn,
+            dn: str,
             filterstr: str = '(objectClass=*)',
             attrlist: Optional[AttrList] = None,
             req_ctrls: Optional[RequestControls] = None,
             timeout: Union[int, float] = -1,
             cache_ttl: Optional[Union[int, float]] = None,
-        ) -> LDAPResult:
+        ) -> SearchResultEntry:
         ldap_res = self.search_s(
             dn,
             _libldap0.SCOPE_BASE,
@@ -1086,7 +1098,7 @@ class LDAPObject:
 
     def read_subschemasubentry_s(
             self,
-            subschemasubentry_dn: None,
+            subschemasubentry_dn: str,
             attrs: Optional[AttrList] = None,
         ) -> EntryStr:
         try:
@@ -1214,8 +1226,6 @@ class ReconnectLDAPObject(LDAPObject):
     )
     __transient_attrs__ = {
         '_cache',
-        '_cache_hits',
-        '_cache_misses',
         '_l',
         '_libldap0_lock',
         '_reconnect_lock',
@@ -1281,7 +1291,7 @@ class ReconnectLDAPObject(LDAPObject):
         for key, val in data.items():
             setattr(self, key, val)
         self._whoami_dn = None
-        self.flush_cache()
+        self._cache = Cache(self._cache_ttl)
         self._last_bind = (
             getattr(LDAPObject, self._last_bind[0]),
             self._last_bind[1],
