@@ -36,6 +36,13 @@ allow bind_v2
 sasl-secprops none
 security ssf=0 simple_bind=0 sasl=0 transport=0
 
+TLSVerifyClient allow
+TLSProtocolMin 3.3
+TLSCACertificateFile tests/tls/ca-chain.pem
+TLSCertificateFile tests/tls/localhost.crt
+TLSCertificateKeyFile tests/tls/localhost.key
+TLSDHParamFile tests/tls/slapd-dhparams.pem
+
 authz-regexp
   "gidnumber=%(root_gid)s\\+uidnumber=%(root_uid)s,cn=peercred,cn=external,cn=auth"
   "%(rootdn)s"
@@ -128,6 +135,13 @@ class SlapdObject:
         self.ldap_uri = "ldap://%s:%d/" % (LOCALHOST, self._port)
         ldapi_path = os.path.join(self.testrundir, 'ldapi')
         self.ldapi_uri = "ldapi://%s" % quote_plus(ldapi_path)
+        self._slapd_version = None
+
+    @property
+    def slapd_version(self):
+        if self._slapd_version is None:
+            self._slapd_version = '.'.join(self.get_slapd_version())
+        return self._slapd_version
 
     def setup_rundir(self):
         """
@@ -429,7 +443,7 @@ class SlapdTestCase(unittest.TestCase):
                 escape_format(escape_filter_str, '(olcSuffix={0})', self.server.suffix),
             )[0].dn_s
 
-    def enable_overlay(self, slapo_entry: EntryStr, schema_files=None):
+    def enable_overlay(self, slapo_entry: EntryStr, schema_files=None, config_dn=None):
         logging.debug('Enable overlay with %r', slapo_entry)
         olc_overlay = slapo_entry['olcOverlay'][0]
         enabled_overlays = [
@@ -459,7 +473,7 @@ class SlapdTestCase(unittest.TestCase):
             self._ldap_conn.add_s(
                 'olcOverlay={{0}}{0},{1}'.format(
                     olc_overlay,
-                    self.mdb_config_dn,
+                    config_dn or self.mdb_config_dn,
                 ),
                 {
                     at: encode_list(avs)
